/*
 * div.c IF 93/09/12
 * 
 *  Float input only, i.e. no div by zero check
 *
 */

#include "ext.h"		// Required for all Max external objects
void *this_class;	   // Required. Global pointing to this class

typedef struct _div	// Data structure for this object
{
	t_object d_ob;
	float d_valleft;	   // Last value from left inlet
	float d_valright;	   // Last value from right inlet
	float d_valtotal;	   // Value to be sent to outlet
	void *d_out;		   // Pointer to outlet, need one for each outlet
} t_div;

// Prototypes for methods: you need a method for each message you want to respond to
void *div_new(float value);
void div_int(t_div *div, long value);	// Method for message "int" in left inlet
void div_in1(t_div *div, long value);	// Method for message "int" in right inlet
void div_flt(t_div *div, float value);	// Method for message "float" in left inlet
void div_fl1(t_div *div, float value);  // Method for message "float" in right inlet
void div_bang(t_div *div);				// Method for bang message
void div_assist(t_div *div, Object *b, long msg, long arg, char *s); //Assistance method

int main(void)			// main receives a copy of the Max function macros table
{
	// set up our class: create a class definition 
	setup((t_messlist **) &this_class, (method)div_new, 0L, (short)sizeof(t_div), 
				0L, A_DEFFLOAT, 0);
		
	addbang((method)div_bang);	// bind method "div_bang" to the "bang" message	
	addint((method)div_int);	// bind method "div_int" to int's received in the left inlet	
	addinx((method)div_in1,1);	// bind method "div_in1" to int's received in the right inlet	
	addfloat((method)div_flt);	// bind method "div_flt" to float's received in the left inlet	
	addftx((method)div_fl1,1);	// bind method "div_fl1" to float's received in the right inlet	
	addmess((method)div_assist, "assist", A_CANT, 0);	
	finder_addclass("All Objects", "div_assist");
	return (0);	
}

/**********************************************************************************
div_new(float value)

inputs:			value -- the float or integer from the typed argument in the object box	
description:	creates a new instance of our class div.  Called once when the external
						object is loaded.
returns:		pointer to new instance
*************************************************************************************/

void *div_new(float value)
{	
	t_div *div;
	
	div = (t_div *)newobject(this_class);	// Create new instance and return a pointer to it
		
	if (value == 0.0)	//error handling - division by 0
		value = 1.0;
	
	div->d_valright = value;	// Initialize the denominator
	div->d_valleft = 0.0;
	div->d_valtotal = 0.0;
	div->d_out = floatout(div); // Create float outlet

	floatin(div, 1); 		// Create the right inlet 

	return(div); // Must return a pointer to the new instance
		
}

/**********************************************************************************
div_int(t_div *a, long value)

inputs:			div	- pointer to t_div object
				value	- value received in the inlet
description:	divides the incoming value by the right value. Stores the new left inlet 
							value as well as the total and outputs the total.
returns:		nothing
***********************************************************************************/

void div_int(t_div *div, long value)
{
	div->d_valleft = value;	// Store the value received in the left inlet 	
	div->d_valtotal = div->d_valleft / div->d_valright; 	// divides the left inlet
															// by the right															
	div_bang(div);			// Call bang method right away since it's the left inlet	
}

/*************************************************************************************
div_in1(t_div *div, long value)

inputs:			div	- pointer to our object
				value	- value received in the inlet
description:	stores the new right value, calculates and stores the new quotient 
returns:		nothing
*************************************************************************************/

void div_in1(t_div *div, long value)
{	
	if (value == 0)				//error handling - division by 0
		value = 1;
	
	div->d_valright = value;	// Store the value	
	div->d_valtotal = div->d_valleft / div->d_valright; // Update new quotient
		
}

/**********************************************************************************
div_flt(t_div *a, float value)

inputs:			div		- pointer to t_div object
				value	- value received in the inlet
description:	divides the right value by the incoming value. Stores the new left inlet 
							value as well as the total and outputs the total.
returns:		nothing
***********************************************************************************/

void div_flt(t_div *div, float value)
{
	div->d_valleft = value;	// Store the value received in the left inlet 
		
	div->d_valtotal = div->d_valleft / div->d_valright;	// divides the left inlet
														// by the right
		
	div_bang(div);			// Call bang method right away since it's the left inlet	
}

/*************************************************************************************
div_fl1(t_div *div, float value)

inputs:			div		- pointer to t_div object
				value	- value received in the inlet
description:	stores the new right value, calculates and stores the new quotient 
returns:		nothing
*************************************************************************************/

void div_fl1(t_div *div, float value)
{	
	if (value == 0.0)			//error handling - division by 0
		value = 1.0;
	
	div->d_valright = value;	// Store the value
	
	div->d_valtotal = div->d_valleft / div->d_valright;	// Update new quotient
		
}
		

/*************************************************************************************
div_bang(t_div *a)

inputs:			div	- pointer to t_div object
description:	method called when bang is received: it outputs the current quotient 
						of the left and right values
returns:		nothing
*************************************************************************************/

void div_bang(t_div *div)
{
	outlet_float(div->d_out, div->d_valtotal);
}

/*************************************************************************************
void div_assist(a, b, msg, arg, s)

inputs: 		a - pointer to t_div object
				b - pointer to the t_div object's box
				msg - specifies whether request for inlet or outlet info
				arg - selected inlet or outlet number
				s - destination for assistance string
description: 	method called when assist message is received: it outputs the correct
				assistance message string to the patcher window
returns: nothing
*************************************************************************************/

void div_assist(t_div *div, Object *b, long msg, long arg, char *s)
			// copy the appropriate message to the destination string
{
	if (msg == ASSIST_INLET)
	{
		switch (arg)
		{
			case 0: sprintf(s, "%s", "Left Operand");
				break;
			case 1: sprintf(s, "%s", "Right Operand");
		}
	}
	else if (msg == ASSIST_OUTLET)
		sprintf(s, "%s", "Result = Left / Right");
}